/*
 * pmvncdef.h - PM VNC Viewer, Global Definitions
 */

#ifndef _PMVNCDEF_H
#define _PMVNCDEF_H

/*
 * Debugging Macro
 */

#ifdef  DEBUG
#define TRACE       printf
#else
#define TRACE
#endif

/*
 * Types required for 'rfbproto.h'
 */

typedef CHAR    INT8   ;
typedef SHORT   INT16  ;
typedef LONG    INT32  ;
typedef UCHAR   CARD8  ;
typedef USHORT  CARD16 ;
typedef ULONG   CARD32 ;

#include "rfbproto.h"

/*
 * Program Name & Profile Path
 */

extern  UCHAR   ProgramVers[] ;
extern  UCHAR   ProgramPath[] ;
extern  UCHAR   ProgramName[] ;
extern  UCHAR   ProfilePath[] ;

/*
 * Common Functions
 */

void    errMessage(PSZ msg) ;
void    dialogAtMouse(HWND hwndDialog, int target) ;

#define swap16(x)   ((((x) & 0xff) << 8) | (((x) >> 8) & 0xff))
#define swap32(x)   ((((x) & 0xff000000) >> 24) | \
                     (((x) & 0x00ff0000) >>  8) | \
		     (((x) & 0x0000ff00) <<  8) | \
		     (((x) & 0x000000ff) << 24))

/*
 * Windows for VNC Viewer
 */

extern  HWND    hwndFrame  ;
extern  HWND    hwndClient ;
extern  HWND    hwndHorz   ;        /* Horizontal Scroll Bar    */
extern  HWND    hwndVert   ;        /* Vertical   Scroll Bar    */

BOOL    winCreate(HAB hab)  ;
void    winDispose(HAB hab) ;

/*
 * Messages from Network Thread (request GUI/Graphics Operation)
 */

#define WM_VNC_FAIL     (WM_USER + 1)       /* error on network     */

#define WM_VNC_AUTH     (WM_USER + 2)       /* authentication       */
#define WM_VNC_INIT     (WM_USER + 3)       /* size/name of RFB     */

#define WM_VNC_BELL     (WM_USER + 4)       /* bell                 */
#define WM_VNC_CLIP     (WM_USER + 5)       /* remote text clip     */

#define WM_VNC_UPDATE   (WM_USER + 6)       /* final update         */

/*
 * Bitmaps used for RFB's backing store
 */

extern  HDC     hdcBitmap ;
extern  HPS     hpsBitmap ;
extern  HBITMAP hbmBitmap ;

BOOL    bmpCreate(HAB hab, PSIZEL sizrfb) ;
void    bmpDispose(void) ;

void    bmpMapSize(SHORT cx, SHORT cy) ;
void    bmpMapHorz(USHORT cmd, SHORT x) ;
void    bmpMapVert(USHORT cmd, SHORT y) ;
void    bmpMapPoint(PPOINTL pt, BOOL toBmp) ;

void    bmpPaint(HPS hps, PRECTL rct) ;

void    bmpUpdate(int x, int y, int w, int h) ;
void    bmpRectDraw(int x, int y, int w, int h, PUCHAR bits) ;
void    bmpRectCopy(int x, int y, int w, int h, int sx, int sy) ;
void    bmpRectFill(int x, int y, int w, int h) ;
/* void    bmpRectColor(ULONG color) ; */
#define bmpRectColor(color) GpiSetColor(hpsBitmap, (color))

/*
 * Mouse Event Handlings
 */

void    mouseInit(HWND hwnd) ;
void    mouseDone(HWND hwnd) ;
void    mouseMove(HWND hwnd, SHORT x, SHORT y) ;
void    mouseEvent(HWND hwnd, ULONG msg, SHORT x, SHORT y) ;
void    mouseTimer(HWND hwnd) ;

/*
 * Keyboard Event Handling
 */

void    keyEvent(HWND hwnd, USHORT flags, USHORT ch, USHORT vk, USHORT sc) ;

/*
 * Networking for VNC Viewer
 */

extern  UCHAR   ServerName[] ;
extern  int     ServerSock   ;

BOOL    netStartup(HAB hab) ;
void    netFinish(HAB hab)  ;

void    netFail(PUCHAR msg) ;
void    netNotify(int ev, int len, PUCHAR p) ;
void    netRect(int ev, int len) ;
void    netUpdate(int x, int y, int w, int h) ;

BOOL    netSend(PUCHAR buf, int len) ;
BOOL    netRecv(PUCHAR buf, int len) ;
void    netDump(PUCHAR buf, int len) ;  /* for DEBUG */

/*
 * Session Setup
 */

BOOL    sessSetup(HAB hab)  ;
BOOL    sessOption(HAB hab) ;

extern  CARD32  SessOptEncoding ;
extern  BOOL    SessOptShared   ;
extern  BOOL    SessOptViewonly ;
extern  BOOL    SessOptUse8bits ;
extern  BOOL    SessOptDeicon   ;

/*
 * RFB Protocol Handling
 */

typedef struct _VNCCTX {
    rfbPixelFormat  rectFormat ;
    BOOL    (*rectRaw)   (rfbRectangle *r) ;
    BOOL    (*rectCopy)  (rfbRectangle *r) ;
    BOOL    (*rectRRE)   (rfbRectangle *r) ;
    BOOL    (*rectCoRRE) (rfbRectangle *r) ;
    BOOL    (*rectTile)  (rfbRectangle *r) ;
} VNCCTX, *PVNCCTX ;

extern  VNCCTX      ctxPix32 ;      /* for 32 bit true color */
extern  VNCCTX      ctxPix8  ;      /* for bgr233            */

extern  PVNCCTX     ctxPix   ;        /* current context  */

BOOL    protoConnInit(void) ;
BOOL    protoSendAuth(void) ;
PUCHAR  protoQueryDesktop(PSIZEL siz) ;

BOOL    protoSendFmtEnc(void) ;
BOOL    protoSendRequest(BOOL inc, PRECTL rect) ;

BOOL    protoDispatch(void) ;

BOOL    protoSendMouEvent(USHORT state, PPOINTL pt) ;
BOOL    protoSendKeyEvent(BOOL down, ULONG key)     ;

/*
 * rectangle drawings (framebuffer updates)
 */

/*
 * Authentication
 */

BOOL    authCypher(HWND hwnd, PUCHAR key) ;

#endif  /* _PMVNCDEF_H */
