/*
 * sess.c - PM VNC Viewer, Session Setup
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INCL_PM
#include <os2.h>

#include "pmvncdef.h"
#include "pmvncres.h"

/*
 * Session Options
 */

CARD32  SessOptEncoding = (CARD32) -1 ;     /* not specified */
BOOL    SessOptShared   = FALSE ;
BOOL    SessOptViewonly = FALSE ;
BOOL    SessOptUse8bits = FALSE ;
BOOL    SessOptDeicon   = FALSE ;

/*
 * set/get Button State
 */

static  void    setButton(HWND hwndDlg, USHORT id, BOOL state)
{
    HWND    hwnd = WinWindowFromID(hwndDlg, id) ;

    WinSendMsg(hwnd, BM_SETCHECK, MPFROMSHORT(state), NULL) ;
}

static  BOOL    getButton(HWND hwndDlg, USHORT id)
{
    HWND    hwnd = WinWindowFromID(hwndDlg, id) ;
    BOOL    state ;

    state = (BOOL) WinSendMsg(hwnd, BM_QUERYCHECK, NULL, NULL) ;
    return state ;
}

/*
 * set/get Session Options
 */

static  BOOL    initialSetup ;

static  void    setOptions(HWND hwndDlg)
{
    setButton(hwndDlg, IDD_OPREF_HEX, SessOptEncoding == rfbEncodingHextile) ;
    setButton(hwndDlg, IDD_OPREF_COR, SessOptEncoding == rfbEncodingCoRRE)   ;
    setButton(hwndDlg, IDD_OPREF_RRE, SessOptEncoding == rfbEncodingRRE)     ;
    setButton(hwndDlg, IDD_OPREF_RAW, SessOptEncoding == rfbEncodingRaw)     ;

    setButton(hwndDlg, IDD_OSET_DEICON, SessOptDeicon)   ;
    setButton(hwndDlg, IDD_OSET_8BITS,  SessOptUse8bits) ;
    setButton(hwndDlg, IDD_OSET_SHARED, SessOptShared)   ;
    setButton(hwndDlg, IDD_OSET_VONLY,  SessOptViewonly) ;

    if (! initialSetup) {
        WinEnableWindow(WinWindowFromID(hwndDlg, IDD_OSET_SHARED), FALSE) ;
        WinEnableWindow(WinWindowFromID(hwndDlg, IDD_OSET_8BITS),  FALSE) ;
    }
}

static  BOOL    getOptions(HWND hwndDlg)
{
    if (getButton(hwndDlg, IDD_OPREF_HEX)) {
        SessOptEncoding = rfbEncodingHextile ;
    } else if (getButton(hwndDlg, IDD_OPREF_COR)) {
        SessOptEncoding = rfbEncodingCoRRE ;
    } else if (getButton(hwndDlg, IDD_OPREF_RRE)) {
        SessOptEncoding = rfbEncodingRRE ;
    } else if (getButton(hwndDlg, IDD_OPREF_RAW)) {
        SessOptEncoding = rfbEncodingRaw ;
    }
    
    if (initialSetup) {
        SessOptShared   = getButton(hwndDlg, IDD_OSET_SHARED) ;
	SessOptUse8bits = getButton(hwndDlg, IDD_OSET_8BITS)  ;
    }
    SessOptDeicon   = getButton(hwndDlg, IDD_OSET_DEICON) ;
    SessOptViewonly = getButton(hwndDlg, IDD_OSET_VONLY)  ;

    return TRUE ;
}

/*
 * procOpts - dialog proc. for Session Options
 */

static MRESULT EXPENTRY procOpts(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    ULONG   ret ;

    switch (msg) {
    
    case WM_INITDLG :
        dialogAtMouse(hwnd, DID_OK) ;
	setOptions(hwnd)   ;
	return (MRESULT) 0 ;

    case WM_COMMAND :
        switch (SHORT1FROMMP(mp1)) {
        case DID_OK :
	    if (getOptions(hwnd) != TRUE) {
	        return (MRESULT) 0 ;
	    }
	    WinDismissDlg(hwnd, DID_OK) ;
	    return (MRESULT) 0 ;
        case DID_CANCEL :
	    WinDismissDlg(hwnd, DID_CANCEL) ;
	    return (MRESULT) 0 ;
        default :
	    return (MRESULT) 0 ;
	}
    }
    return WinDefDlgProc(hwnd, msg, mp1, mp2) ;
}

/*
 * set/getServer - set/get Server Spec.
 */

static  void    setServer(HWND hwndDlg)
{
    HWND    hwnd = WinWindowFromID(hwndDlg, IDD_SHOST) ;
    
    WinSendMsg(hwnd, EM_SETTEXTLIMIT, MPFROMSHORT(128), NULL) ;
    WinSetWindowText(hwnd, ServerName) ;
}

static  BOOL    getServer(HWND hwndDlg)
{
    HWND    hwnd = WinWindowFromID(hwndDlg, IDD_SHOST) ;
    UCHAR   server[128] ;
    
    WinQueryWindowText(hwnd, 128, server) ;

    if (strlen(server) == 0) {
        return FALSE ;
    }
    if (strchr(server, ':') == NULL) {
        return FALSE ;
    }
    strcpy(ServerName, server) ;
    return TRUE ;
}

/*
 * procSess - dialog proc. for Session Setup
 */

static MRESULT EXPENTRY procSess(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    ULONG   ret ;

    switch (msg) {
    
    case WM_INITDLG :
        dialogAtMouse(hwnd, DID_OK) ;
	setServer(hwnd)    ;
	return (MRESULT) 0 ;

    case WM_COMMAND :
        switch (SHORT1FROMMP(mp1)) {
        case DID_OK :
	    if (getServer(hwnd) != TRUE) {
	        return (MRESULT) 0 ;
	    }
	    WinDismissDlg(hwnd, DID_OK) ;
	    return (MRESULT) 0 ;
        case DID_CANCEL :
	    WinDismissDlg(hwnd, DID_CANCEL) ;
	    return (MRESULT) 0 ;
        case IDD_SOPTS :
            ret = WinDlgBox(HWND_DESKTOP, hwndClient,
                        procOpts, NULLHANDLE, IDD_OPTS, NULL) ;
	    return (MRESULT) 0 ;
        default :
	    return (MRESULT) 0 ;
	}
    }
    return WinDefDlgProc(hwnd, msg, mp1, mp2) ;
}

/*
 * sessSetup - Session Setup, entry
 */

BOOL    sessSetup(HAB hab)
{
    ULONG   ret ;
    
    initialSetup = TRUE ;
    
    ret = WinDlgBox(HWND_DESKTOP, hwndClient,
                procSess, NULLHANDLE, IDD_SESS, NULL) ;
    return (ret == DID_OK) ? TRUE : FALSE ;
}

/*
 * sessOption - Change Session option during a session
 */

BOOL    sessOption(HAB hab)
{
    ULONG   ret ;

    initialSetup = FALSE ;
    
    ret = WinDlgBox(HWND_DESKTOP, hwndClient,
                procOpts, NULLHANDLE, IDD_OPTS, NULL) ;
    return (ret == DID_OK) ? TRUE : FALSE ;
}
