/*
 * window.c - PM VNC Viewer, Window Manipulations
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INCL_PM
#include <os2.h>

#include "pmvncdef.h"
#include "pmvncres.h"

/*
 * window handles and their size & position
 */
 
HWND    hwndFrame  = NULLHANDLE ;
HWND    hwndClient = NULLHANDLE ;
HWND    hwndHorz   = NULLHANDLE ;
HWND    hwndVert   = NULLHANDLE ;

static  SWP     swpFrame  = { 0 } ; /* size of frame  window        */
static  SWP     swpClient = { 0 } ; /* size of client window        */
static  SIZEL   sizMax    = { 0 } ; /* max size of frame            */
static  SIZEL   sizRfb    = { 0 } ; /* size of remote screen        */

static  ULONG   sv_cxScreen     ;   /* width  of screen             */
static  ULONG   sv_cyScreen     ;   /* height of screen             */
static  ULONG   sv_cxVScroll    ;   /* width  of vert. scroll bar   */
static  ULONG   sv_cyHScroll    ;   /* height of horz. scroll bar   */
static  ULONG   sv_cyTitleBar   ;   /* height of title bar          */
static  ULONG   sv_cxSizeBorder ;   /* width  of sizing border      */
static  ULONG   sv_cySizeBorder ;   /* height of sizing border      */

/*
 * winSetup - adjust window with Remote Frame Buffer
 */

static  BOOL    winSetup(void)
{
    PUCHAR  p ;
    UCHAR   title[256] ;
    
    if (hwndFrame == NULLHANDLE || hwndClient == NULLHANDLE) {
        errMessage("winSetup - no windows") ;
	return FALSE ;
    }
    
    /*
     * query name & size of remote frame buffer
     */
     
    if ((p = protoQueryDesktop(&sizRfb)) == NULL) {
        strcpy(title, ProgramVers) ;
    } else if (strlen(p) < 8) {
        sprintf(title, "%s [%s]", ProgramVers, p) ;
    } else {
        strcpy(title, p) ;
    }

    /*
     * create bitmap for given remote frame buffer
     */
     
    if (bmpCreate(WinQueryAnchorBlock(hwndFrame), &sizRfb) != TRUE) {
        errMessage("winSetup - failed to setup bitmap") ;
	return FALSE ;
    } 
    
    /*
     * calc. size (max & current) & position (initial) of frame window
     */

    sizMax.cx = sizRfb.cx + (sv_cxSizeBorder * 2) + sv_cxVScroll - 1 ;
    sizMax.cy = sizRfb.cy + (sv_cySizeBorder * 2) +
                                    sv_cyTitleBar + sv_cyHScroll - 1 ;
    if (sizMax.cx <= sv_cxScreen) {
        swpFrame.cx = sizMax.cx ;
        swpFrame.x = (sv_cxScreen - sizMax.cx) / 2 ;
    } else {
        swpFrame.cx = sv_cxScreen ;
        swpFrame.x  = 0 ;
    }
    if (sizMax.cy <= sv_cyScreen) {
        swpFrame.cy = sizMax.cy ;
        swpFrame.y = (sv_cyScreen - sizMax.cy) / 2 ;
    } else {
        swpFrame.cy = sv_cyScreen ;
        swpFrame.y = 0 ;
    }     

    /*
     * show window
     */
     
    WinSetWindowText(hwndFrame, title) ;
    WinSetWindowPos(hwndFrame, NULLHANDLE,
        swpFrame.x, swpFrame.y, swpFrame.cx, swpFrame.cy,
	(SWP_MOVE | SWP_SIZE | SWP_SHOW)) ;

    return TRUE ;
}

/*
 * procFrame - subclassed frame window
 */

static  PFNWP   pfnFrame ;

static MRESULT EXPENTRY procFrame(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    PSWP    pswp ;
    
    switch (msg) {

    case WM_WINDOWPOSCHANGED :
        pswp = (PSWP) PVOIDFROMMP(mp1) ;
	swpFrame = *pswp ;
	break ;

    case WM_ADJUSTWINDOWPOS :
        pswp = (PSWP) PVOIDFROMMP(mp1) ;
	if ((pswp->fl & SWP_MAXIMIZE) && (pswp->fl & SWP_RESTORE) == 0) {
	    pswp->fl &= ~SWP_MAXIMIZE ;
	    pswp->fl |=  (SWP_SIZE | SWP_MOVE) ;
	    pswp->cx = sizMax.cx ;
	    pswp->cy = sizMax.cy ;
	    pswp->y -= (sizMax.cy - swpFrame.cy) ;
	}

	/*
	 * limit window sizing
	 */
	if ((pswp->fl & SWP_SIZE) && (pswp->cx > sizMax.cx)) {
            if (pswp->x == swpFrame.x) {    /* Enlarged to Right */
	        pswp->x += (pswp->cx - sizMax.cx) ;
		pswp->fl |= SWP_MOVE ;
	    }
	    pswp->cx = sizMax.cx ;
	}
	if ((pswp->fl & SWP_SIZE) && (pswp->cy > sizMax.cy)) {
	    if (pswp->y == swpFrame.y) {    /* Enlaged to Upper */
	        pswp->y += (pswp->cy - sizMax.cy) ;
		pswp->fl |= SWP_MOVE ;
	    }
	    pswp->cy = sizMax.cy ;
	}
	break ;
    }
    return (*pfnFrame) (hwnd, msg, mp1, mp2) ;
}

/*
 * procClient - client window procedure
 */

static MRESULT EXPENTRY procClient(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    HPS     hps ;
    RECTL   rct ;
    SHORT   cx, cy ;

    switch (msg) {

    case WM_CREATE :
        mouseInit(hwnd) ;
	return (MRESULT) 0 ;
    case WM_DESTROY :
        mouseDone(hwnd) ;
	return (MRESULT) 0 ;	
	
    case WM_SIZE  :
        cx = SHORT1FROMMP(mp2) ;
	cy = SHORT2FROMMP(mp2) ;
	bmpMapSize(cx, cy) ;
	return (MRESULT) 0 ;
	
    case WM_HSCROLL :
        bmpMapHorz(SHORT2FROMMP(mp2), SHORT1FROMMP(mp2)) ;
	return (MRESULT) 0 ;
    case WM_VSCROLL :
        bmpMapVert(SHORT2FROMMP(mp2), SHORT1FROMMP(mp2)) ;
        return (MRESULT) 0 ;
    
    case WM_PAINT :
        hps = WinBeginPaint(hwnd, NULLHANDLE, &rct) ;
        bmpPaint(hps, &rct) ;
	WinEndPaint(hps) ;
	return (MRESULT) 0 ;

    case WM_MOUSEMOVE     :
        mouseMove(hwnd, SHORT1FROMMP(mp1), SHORT2FROMMP(mp1)) ;
	return (MRESULT) 0 ;
    case WM_BUTTON1DOWN   :
    case WM_BUTTON2DOWN   :
    case WM_BUTTON3DOWN   :
    case WM_BUTTON1UP     :
    case WM_BUTTON2UP     :
    case WM_BUTTON3UP     :
    case WM_BUTTON1CLICK  :
    case WM_BUTTON2CLICK  :
    case WM_BUTTON3CLICK  :
    case WM_BUTTON1DBLCLK :
    case WM_BUTTON2DBLCLK :
    case WM_BUTTON3DBLCLK :
        mouseEvent(hwnd, msg, SHORT1FROMMP(mp1), SHORT2FROMMP(mp1)) ;
        return WinDefWindowProc(hwnd, msg, mp1, mp2) ;

    case WM_CHAR :
        keyEvent(hwnd, 
	    SHORT1FROMMP(mp1), SHORT1FROMMP(mp2), 
	    SHORT2FROMMP(mp2), SHORT2FROMMP(mp1)) ;
        return (MRESULT) 0 ;

    case WM_TIMER :
        mouseTimer(hwnd) ;
	return (MRESULT) 0 ;
	
    /*
     * local messages
     */

    case WM_VNC_FAIL :
        errMessage((PUCHAR) mp2) ;
	WinPostMsg(hwnd, WM_CLOSE, NULL, NULL) ;
	return (MRESULT) 0 ;
    
    case WM_VNC_AUTH :
        authCypher(hwnd, (PUCHAR) mp2) ;
        protoSendAuth() ;
        return (MRESULT) 0 ;
    
    case WM_VNC_INIT :
        if (winSetup() != TRUE) {
            WinPostMsg(hwnd, WM_CLOSE, NULL, NULL) ;
	    return (MRESULT) 0 ;
        }
        return (MRESULT) 0 ;
 
    case WM_VNC_BELL :
        WinAlarm(HWND_DESKTOP, WA_NOTE) ;
	return (MRESULT) 0 ;
	
    case WM_VNC_CLIP :
        WinAlarm(HWND_DESKTOP, WA_WARNING) ;
        return (MRESULT) 0 ;
    
    case WM_VNC_UPDATE :
        bmpUpdate(SHORT1FROMMP(mp1), SHORT2FROMMP(mp1),
	                  SHORT1FROMMP(mp2), SHORT2FROMMP(mp2)) ;
        return (MRESULT) 0 ;
    }
    return WinDefWindowProc(hwnd, msg, mp1, mp2) ;
}

/*
 * winCreate - create windows
 *      windows are first invisible and 0 sized.  They are sized to
 *      RFB on received ServerInit message from VNC server.
 */

#define VNCPROC "vncViewer"

BOOL    winCreate(HAB hab)
{
    ULONG   flFrameStyle ;
    ULONG   flFrameFlags ;
    
    if (hwndFrame != NULLHANDLE || hwndClient != NULLHANDLE) {
        return FALSE ;
    }
    
    /*
     * create standard window (set)
     */

    flFrameStyle = 0 ;
    flFrameFlags = FCF_TASKLIST | FCF_TITLEBAR | FCF_ICON | FCF_SYSMENU |
                   FCF_SIZEBORDER | FCF_MINBUTTON | FCF_MAXBUTTON |
                   FCF_HORZSCROLL | FCF_VERTSCROLL ;

    WinRegisterClass(hab, VNCPROC, procClient, CS_SIZEREDRAW, 0) ;
    
    hwndFrame = WinCreateStdWindow(
            HWND_DESKTOP,           /* parent window handle     */
	    flFrameStyle,           /* frame window styles      */
	    &flFrameFlags,          /* create flags             */
	    VNCPROC,                /* client window class      */
	    ProgramName,            /* as title                 */
	    0,                      /* client window styles     */
	    NULLHANDLE,             /* resource (internal)      */
	    ID_PMVNC,               /* window ID                */
	    &hwndClient) ;          /* client window            */

    if (hwndFrame == NULLHANDLE || hwndClient == NULLHANDLE) {
        return FALSE ;
    }

    pfnFrame = WinSubclassWindow(hwndFrame, procFrame) ;

    WinStartTimer(hab, hwndClient, 1, 50) ;
    
    WinQueryWindowPos(hwndFrame,  &swpFrame)  ;
    WinQueryWindowPos(hwndClient, &swpClient) ;
    
    hwndHorz = WinWindowFromID(hwndFrame, FID_HORZSCROLL) ;
    hwndVert = WinWindowFromID(hwndFrame, FID_VERTSCROLL) ;

    sv_cxScreen     = WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN)     ;
    sv_cyScreen     = WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN)     ;
    sv_cxVScroll    = WinQuerySysValue(HWND_DESKTOP, SV_CXVSCROLL)    ;
    sv_cyHScroll    = WinQuerySysValue(HWND_DESKTOP, SV_CYHSCROLL)    ;
    sv_cyTitleBar   = WinQuerySysValue(HWND_DESKTOP, SV_CYTITLEBAR)   ;
    sv_cxSizeBorder = WinQuerySysValue(HWND_DESKTOP, SV_CXSIZEBORDER) ;
    sv_cySizeBorder = WinQuerySysValue(HWND_DESKTOP, SV_CYSIZEBORDER) ;

    return TRUE ;
}

/*
 * winDispose - dispose windows
 */

void    winDispose(HAB hab)
{
    WinStopTimer(hab, hwndClient, 1) ;
    
    bmpDispose() ;
    
    if (hwndClient != NULLHANDLE) {
        WinDestroyWindow(hwndClient) ;
	hwndClient = NULLHANDLE ;
    }
    if (hwndFrame != NULLHANDLE) {
        WinDestroyWindow(hwndFrame) ;
	hwndFrame = NULLHANDLE ;
    }
}
